/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_CS_PROXY_GEN_CB_H_
#define IDLC_CS_GEN_CS_PROXY_GEN_CB_H_

const char CB_DATA_MEMBERS[] =
R"__cs_cb(            public event EventHandler Connected;
            public event EventHandler Disconnected;
            public event EventHandler Rejected;

            private bool _online = false;
            private string _appId;
            private Object _lock = new Object();
            private List<CallbackBase> _delegateList = new List<CallbackBase>();
)__cs_cb";

const char CB_EVENT_METHODS[] =
R"__cs_cb(
            protected override void OnConnectedEvent(string endPoint, string portName, Port port)
            {
                _online = true;
                Connected?.Invoke(this, null);
            }

            protected override void OnDisconnectedEvent(string endPoint, string portName)
            {
                _online = false;
                Disconnected?.Invoke(this, null);
            }

            protected override void OnRejectedEvent(string endPoint, string portName)
            {
                Rejected?.Invoke(this, null);
            }

            private void ProcessReceivedEvent(Parcel parcel)
            {
                int id = parcel.ReadInt();
                int seqId = parcel.ReadInt();
                bool once = parcel.ReadBool();

                foreach (var i in _delegateList)
                {
                    if ((int)i.Id == id && i.SeqId == seqId)
                    {
                        i.OnReceivedEvent(parcel);
                        if (i.Once)
                            _delegateList.Remove(i);
                        break;
                    }
                }
            }

            protected override void OnReceivedEvent(string endPoint, string portName)
            {
                Parcel parcelReceived;

                parcelReceived = new Parcel(CallbackPort);

                using (parcelReceived)
                {
                    int cmd = parcelReceived.ReadInt();
                    if (cmd != (int)MethodId.__Callback)
                    {
                        return;
                    }

                    ProcessReceivedEvent(parcelReceived);
                }
            }

            private void ConsumeCommand(out Parcel parcel, Port port)
            {
                do
                {
                    var p = new Parcel(port);

                    int cmd = p.ReadInt();
                    if (cmd == (int)MethodId.__Result)
                    {
                        parcel = p;
                        return;
                    }

                    p.Dispose();
                    parcel = null;
                } while (true);
            }
)__cs_cb";

const char CB_CONNECT_METHOD[] =
R"__cs_cb(
            /// <summary>
            /// Connects to the service app.
            /// </summary>
            /// <privilege>http://tizen.org/privilege/appmanager.launch</privilege>
            /// <privilege>http://tizen.org/privilege/datasharing</privilege>
            /// <exception cref="InvalidIDException">
            /// Thrown when the appid to connect is invalid.
            /// </exception>
            /// <exception cref="InvalidIOException">
            /// Thrown when internal I/O error happen.
            /// </exception>
            /// <exception cref="PermissionDeniedException">
            /// Thrown when the permission is denied.
            /// </exception>
            /// <remark> If you want to use this method, you must add privileges.</remark>
            public void Connect()
            {
                Connect(_appId, "$$");
            }

            /// <summary>
            /// Disposes delegate objects in this interface
            /// </summary>
            /// <param name="tag">The tag string from delegate object</param>
            void DisposeCallback(string tag)
            {
                foreach (var i in _delegateList)
                {
                    if (i.Tag.Equals(tag))
                    {
                        _delegateList.Remove(i);
                        return;
                    }
                }
            }
)__cs_cb";

const char CB_INVOCATION_PRE[] =
R"__cs_cb(                if (!_online)
                    throw new NotConnectedSocketException();

                using (Parcel p = new Parcel())
                {
$$
		}
)__cs_cb";

const char CB_INVOCATION_MID[] =
R"__cs_cb(                        // Send
                        p.Send(Port);
)__cs_cb";

#endif  // IDLC_CS_GEN_CS_PROXY_GEN_CB_H_
