/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/element.h"

class ElementTest : public testing::Test {
 public:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(ElementTest, Element_Contstructor) {
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  delete elm;
}

TEST_F(ElementTest, Element_GetID) {
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  EXPECT_EQ(elm->GetID(), "test");
  delete elm;
}

TEST_F(ElementTest, Element_GetType) {
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  EXPECT_EQ(elm->GetType().ToString(), "int");
  delete elm;
}

TEST_F(ElementTest, Element_GetLine) {
  unsigned line = __LINE__;
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", line);
  EXPECT_NE(elm, nullptr);
  EXPECT_EQ(elm->GetLine(), line);
  delete elm;
}

TEST_F(ElementTest, Element_GetComments) {
  std::string comments = "Test comments";
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), comments, __LINE__);
  EXPECT_NE(elm, nullptr);
  EXPECT_EQ(elm->GetComments(), comments);
  delete elm;
}

class ElementsTest : public testing::Test {
 public:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(ElementsTest, Elements_Constructor) {
  tidl::Elements* elms = new tidl::Elements();
  EXPECT_NE(elms, nullptr);
  delete elms;
}

TEST_F(ElementsTest, Elements_Add) {
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  tidl::Elements* elms = new tidl::Elements();
  EXPECT_NE(elms, nullptr);
  elms->Add(elm);

  bool flag = false;
  for (auto& elm : elms->GetElms()) {
    if (elm->GetID() == "test" &&
        elm->GetType().ToString() == "int")
      flag = true;
  }
  EXPECT_EQ(flag, true);
  delete elms;
}

TEST_F(ElementsTest, Elements_Getelms) {
  tidl::Elements* elms = new tidl::Elements();
  EXPECT_NE(elms, nullptr);
  elms->Add(new tidl::Element("test1", new tidl::BaseType("int", ""),
      "", __LINE__));
  elms->Add(new tidl::Element("test2", new tidl::BaseType("char *", ""),
      "", __LINE__));

  int count = 0;
  for (auto& elm : elms->GetElms()) {
    if (elm->GetID() == "test1" &&
        elm->GetType().ToString() == "int")
      count++;
    if (elm->GetID() == "test2" &&
        elm->GetType().ToString() == "char *")
      count++;
  }
  EXPECT_EQ(count, 2);
  delete elms;
}

TEST_F(ElementsTest, Elements_Exist) {
  tidl::Element* elm = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  tidl::Elements* elms = new tidl::Elements();
  EXPECT_NE(elms, nullptr);
  elms->Add(elm);
  tidl::Element* elm2 = new tidl::Element("test",
      new tidl::BaseType("int", ""), "", __LINE__);
  EXPECT_NE(elm, nullptr);
  EXPECT_EQ(elms->Exist(elm2), true);
  delete elm2;
  delete elms;
}
