/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/parser.h"

extern int gargc;
extern char** gargv;

class ParserTest : public testing::Test {
 public:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(ParserTest, Parser_Constructor) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);
  delete ps;
}

TEST_F(ParserTest, Parser_Scanner) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);
  EXPECT_NE(ps->Scanner(), nullptr);
  delete ps;
}

TEST_F(ParserTest, Parser_SetDoc) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);
  ps->SetDoc(new tidl::Document());
  EXPECT_NE(ps->GetDoc(), nullptr);
  delete ps;
}

TEST_F(ParserTest, Parser_GetDoc) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);
  ps->SetDoc(new tidl::Document());
  EXPECT_NE(ps->GetDoc(), nullptr);
  delete ps;
}

TEST_F(ParserTest, Parser_Parse) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);
  std::string input = "interface ParserTest { int test(in int a); }";
  EXPECT_EQ(ps->Parse(input), true);
  delete ps;
}

TEST_F(ParserTest, Parser_ParseFromFile) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);

  std::string file = "test.tidl";
  if (gargc > 1)
    file = gargv[1];
  EXPECT_EQ(ps->ParseFromFile(file), true);
  delete ps;
}

TEST_F(ParserTest, Parser_ParseFromFile_N) {
  tidl::Parser* ps = new tidl::Parser();
  EXPECT_NE(ps, nullptr);

  std::string file = "test_error.tidl";
  if (gargc > 2)
    file = gargv[2];
  EXPECT_EQ(ps->ParseFromFile(file), false);
  delete ps;
}
